# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .feedback import (
    FeedbackResource,
    AsyncFeedbackResource,
    FeedbackResourceWithRawResponse,
    AsyncFeedbackResourceWithRawResponse,
    FeedbackResourceWithStreamingResponse,
    AsyncFeedbackResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.agents.conversations import response_create_params
from .....types.agents.conversations.response_create_response import ResponseCreateResponse
from .....types.agents.conversations.response_retrieve_response import ResponseRetrieveResponse

__all__ = ["ResponseResource", "AsyncResponseResource"]


class ResponseResource(SyncAPIResource):
    @cached_property
    def feedback(self) -> FeedbackResource:
        return FeedbackResource(self._client)

    @cached_property
    def with_raw_response(self) -> ResponseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ResponseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResponseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return ResponseResourceWithStreamingResponse(self)

    def create(
        self,
        conversation_id: str,
        *,
        agent_id: str,
        message: str,
        stream: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResponseCreateResponse:
        """
        Create response (sync or streamed)

        Args:
          stream: If true and the client sets `Accept: text/event-stream`, the API returns an SSE
              stream instead of a single JSON body.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return self._post(
            f"/agents/{agent_id}/conversations/{conversation_id}/response",
            body=maybe_transform(
                {
                    "message": message,
                    "stream": stream,
                },
                response_create_params.ResponseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ResponseCreateResponse,
        )

    def retrieve(
        self,
        response_id: str,
        *,
        agent_id: str,
        conversation_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResponseRetrieveResponse:
        """
        Get a response

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        if not response_id:
            raise ValueError(f"Expected a non-empty value for `response_id` but received {response_id!r}")
        return self._get(
            f"/agents/{agent_id}/conversations/{conversation_id}/response/{response_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ResponseRetrieveResponse,
        )


class AsyncResponseResource(AsyncAPIResource):
    @cached_property
    def feedback(self) -> AsyncFeedbackResource:
        return AsyncFeedbackResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncResponseResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncResponseResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResponseResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncResponseResourceWithStreamingResponse(self)

    async def create(
        self,
        conversation_id: str,
        *,
        agent_id: str,
        message: str,
        stream: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResponseCreateResponse:
        """
        Create response (sync or streamed)

        Args:
          stream: If true and the client sets `Accept: text/event-stream`, the API returns an SSE
              stream instead of a single JSON body.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return await self._post(
            f"/agents/{agent_id}/conversations/{conversation_id}/response",
            body=await async_maybe_transform(
                {
                    "message": message,
                    "stream": stream,
                },
                response_create_params.ResponseCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ResponseCreateResponse,
        )

    async def retrieve(
        self,
        response_id: str,
        *,
        agent_id: str,
        conversation_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ResponseRetrieveResponse:
        """
        Get a response

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        if not response_id:
            raise ValueError(f"Expected a non-empty value for `response_id` but received {response_id!r}")
        return await self._get(
            f"/agents/{agent_id}/conversations/{conversation_id}/response/{response_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ResponseRetrieveResponse,
        )


class ResponseResourceWithRawResponse:
    def __init__(self, response: ResponseResource) -> None:
        self._response = response

        self.create = to_raw_response_wrapper(
            response.create,
        )
        self.retrieve = to_raw_response_wrapper(
            response.retrieve,
        )

    @cached_property
    def feedback(self) -> FeedbackResourceWithRawResponse:
        return FeedbackResourceWithRawResponse(self._response.feedback)


class AsyncResponseResourceWithRawResponse:
    def __init__(self, response: AsyncResponseResource) -> None:
        self._response = response

        self.create = async_to_raw_response_wrapper(
            response.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            response.retrieve,
        )

    @cached_property
    def feedback(self) -> AsyncFeedbackResourceWithRawResponse:
        return AsyncFeedbackResourceWithRawResponse(self._response.feedback)


class ResponseResourceWithStreamingResponse:
    def __init__(self, response: ResponseResource) -> None:
        self._response = response

        self.create = to_streamed_response_wrapper(
            response.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            response.retrieve,
        )

    @cached_property
    def feedback(self) -> FeedbackResourceWithStreamingResponse:
        return FeedbackResourceWithStreamingResponse(self._response.feedback)


class AsyncResponseResourceWithStreamingResponse:
    def __init__(self, response: AsyncResponseResource) -> None:
        self._response = response

        self.create = async_to_streamed_response_wrapper(
            response.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            response.retrieve,
        )

    @cached_property
    def feedback(self) -> AsyncFeedbackResourceWithStreamingResponse:
        return AsyncFeedbackResourceWithStreamingResponse(self._response.feedback)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncConversationsCursor, AsyncConversationsCursor
from ...._base_client import AsyncPaginator, make_request_options
from ....types.agents import conversation_list_params, conversation_create_params
from .response.response import (
    ResponseResource,
    AsyncResponseResource,
    ResponseResourceWithRawResponse,
    AsyncResponseResourceWithRawResponse,
    ResponseResourceWithStreamingResponse,
    AsyncResponseResourceWithStreamingResponse,
)
from ....types.agents.inconvo_conversation import InconvoConversation
from ....types.agents.conversation_list_response import ConversationListResponse
from ....types.agents.conversation_create_response import ConversationCreateResponse

__all__ = ["ConversationsResource", "AsyncConversationsResource"]


class ConversationsResource(SyncAPIResource):
    @cached_property
    def response(self) -> ResponseResource:
        return ResponseResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConversationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ConversationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConversationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return ConversationsResourceWithStreamingResponse(self)

    def create(
        self,
        agent_id: str,
        *,
        user_identifier: str,
        user_context: Dict[str, Union[str, float, bool]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConversationCreateResponse:
        """
        Create conversation

        Args:
          user_identifier:
              Unique identifier for the end-user (1-256 chars). Allowed characters:
              alphanumeric, underscore, hyphen, period, at symbol.

          user_context: Optional context key-values for additional filtering/tenancy. Required when User
              Context status is ENABLED. Must be omitted when User Context status is DISABLED.
              Requests fail when User Context is UNSET.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._post(
            f"/agents/{agent_id}/conversations",
            body=maybe_transform(
                {
                    "user_identifier": user_identifier,
                    "user_context": user_context,
                },
                conversation_create_params.ConversationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConversationCreateResponse,
        )

    def retrieve(
        self,
        conversation_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InconvoConversation:
        """
        Retrieve conversation

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return self._get(
            f"/agents/{agent_id}/conversations/{conversation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InconvoConversation,
        )

    def list(
        self,
        agent_id: str,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        user_context: Dict[str, Union[str, float, bool]] | Omit = omit,
        user_identifier: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncConversationsCursor[ConversationListResponse]:
        """
        List conversations

        Args:
          user_context: Arbitrary userContext filters, encoded as a deep object. Example: GET
              /conversations?userContext[userId]=42&userContext[orgId]=12

          user_identifier: Filter by user identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get_api_list(
            f"/agents/{agent_id}/conversations",
            page=SyncConversationsCursor[ConversationListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "user_context": user_context,
                        "user_identifier": user_identifier,
                    },
                    conversation_list_params.ConversationListParams,
                ),
            ),
            model=ConversationListResponse,
        )


class AsyncConversationsResource(AsyncAPIResource):
    @cached_property
    def response(self) -> AsyncResponseResource:
        return AsyncResponseResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConversationsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncConversationsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConversationsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncConversationsResourceWithStreamingResponse(self)

    async def create(
        self,
        agent_id: str,
        *,
        user_identifier: str,
        user_context: Dict[str, Union[str, float, bool]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConversationCreateResponse:
        """
        Create conversation

        Args:
          user_identifier:
              Unique identifier for the end-user (1-256 chars). Allowed characters:
              alphanumeric, underscore, hyphen, period, at symbol.

          user_context: Optional context key-values for additional filtering/tenancy. Required when User
              Context status is ENABLED. Must be omitted when User Context status is DISABLED.
              Requests fail when User Context is UNSET.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._post(
            f"/agents/{agent_id}/conversations",
            body=await async_maybe_transform(
                {
                    "user_identifier": user_identifier,
                    "user_context": user_context,
                },
                conversation_create_params.ConversationCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConversationCreateResponse,
        )

    async def retrieve(
        self,
        conversation_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InconvoConversation:
        """
        Retrieve conversation

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        return await self._get(
            f"/agents/{agent_id}/conversations/{conversation_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InconvoConversation,
        )

    def list(
        self,
        agent_id: str,
        *,
        cursor: str | Omit = omit,
        limit: int | Omit = omit,
        user_context: Dict[str, Union[str, float, bool]] | Omit = omit,
        user_identifier: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ConversationListResponse, AsyncConversationsCursor[ConversationListResponse]]:
        """
        List conversations

        Args:
          user_context: Arbitrary userContext filters, encoded as a deep object. Example: GET
              /conversations?userContext[userId]=42&userContext[orgId]=12

          user_identifier: Filter by user identifier

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get_api_list(
            f"/agents/{agent_id}/conversations",
            page=AsyncConversationsCursor[ConversationListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "user_context": user_context,
                        "user_identifier": user_identifier,
                    },
                    conversation_list_params.ConversationListParams,
                ),
            ),
            model=ConversationListResponse,
        )


class ConversationsResourceWithRawResponse:
    def __init__(self, conversations: ConversationsResource) -> None:
        self._conversations = conversations

        self.create = to_raw_response_wrapper(
            conversations.create,
        )
        self.retrieve = to_raw_response_wrapper(
            conversations.retrieve,
        )
        self.list = to_raw_response_wrapper(
            conversations.list,
        )

    @cached_property
    def response(self) -> ResponseResourceWithRawResponse:
        return ResponseResourceWithRawResponse(self._conversations.response)


class AsyncConversationsResourceWithRawResponse:
    def __init__(self, conversations: AsyncConversationsResource) -> None:
        self._conversations = conversations

        self.create = async_to_raw_response_wrapper(
            conversations.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            conversations.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            conversations.list,
        )

    @cached_property
    def response(self) -> AsyncResponseResourceWithRawResponse:
        return AsyncResponseResourceWithRawResponse(self._conversations.response)


class ConversationsResourceWithStreamingResponse:
    def __init__(self, conversations: ConversationsResource) -> None:
        self._conversations = conversations

        self.create = to_streamed_response_wrapper(
            conversations.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            conversations.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            conversations.list,
        )

    @cached_property
    def response(self) -> ResponseResourceWithStreamingResponse:
        return ResponseResourceWithStreamingResponse(self._conversations.response)


class AsyncConversationsResourceWithStreamingResponse:
    def __init__(self, conversations: AsyncConversationsResource) -> None:
        self._conversations = conversations

        self.create = async_to_streamed_response_wrapper(
            conversations.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            conversations.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            conversations.list,
        )

    @cached_property
    def response(self) -> AsyncResponseResourceWithStreamingResponse:
        return AsyncResponseResourceWithStreamingResponse(self._conversations.response)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .feedback import Feedback as Feedback
from .feedback_create_params import FeedbackCreateParams as FeedbackCreateParams
from .feedback_update_params import FeedbackUpdateParams as FeedbackUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ConversationListResponse"]


class ConversationListResponse(BaseModel):
    id: str

    created_at: datetime = FieldInfo(alias="createdAt")

    title: Optional[str] = None

    user_context: Optional[Dict[str, Union[str, float, bool]]] = FieldInfo(alias="userContext", default=None)
    """User context values (null when user context is disabled)"""

    user_identifier: Optional[str] = FieldInfo(alias="userIdentifier", default=None)
    """Unique identifier for the end-user (may be null for legacy conversations)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .agents import (
    AgentsResource,
    AsyncAgentsResource,
    AgentsResourceWithRawResponse,
    AsyncAgentsResourceWithRawResponse,
    AgentsResourceWithStreamingResponse,
    AsyncAgentsResourceWithStreamingResponse,
)

__all__ = [
    "AgentsResource",
    "AsyncAgentsResource",
    "AgentsResourceWithRawResponse",
    "AsyncAgentsResourceWithRawResponse",
    "AgentsResourceWithStreamingResponse",
    "AsyncAgentsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .data_summary import (
    DataSummaryResource,
    AsyncDataSummaryResource,
    DataSummaryResourceWithRawResponse,
    AsyncDataSummaryResourceWithRawResponse,
    DataSummaryResourceWithStreamingResponse,
    AsyncDataSummaryResourceWithStreamingResponse,
)
from .datasets.datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .mcp_servers.mcp_servers import (
    McpServersResource,
    AsyncMcpServersResource,
    McpServersResourceWithRawResponse,
    AsyncMcpServersResourceWithRawResponse,
    McpServersResourceWithStreamingResponse,
    AsyncMcpServersResourceWithStreamingResponse,
)
from .conversations.conversations import (
    ConversationsResource,
    AsyncConversationsResource,
    ConversationsResourceWithRawResponse,
    AsyncConversationsResourceWithRawResponse,
    ConversationsResourceWithStreamingResponse,
    AsyncConversationsResourceWithStreamingResponse,
)

__all__ = ["AgentsResource", "AsyncAgentsResource"]


class AgentsResource(SyncAPIResource):
    @cached_property
    def data_summary(self) -> DataSummaryResource:
        return DataSummaryResource(self._client)

    @cached_property
    def conversations(self) -> ConversationsResource:
        return ConversationsResource(self._client)

    @cached_property
    def datasets(self) -> DatasetsResource:
        return DatasetsResource(self._client)

    @cached_property
    def mcp_servers(self) -> McpServersResource:
        return McpServersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AgentsResourceWithStreamingResponse(self)


class AsyncAgentsResource(AsyncAPIResource):
    @cached_property
    def data_summary(self) -> AsyncDataSummaryResource:
        return AsyncDataSummaryResource(self._client)

    @cached_property
    def conversations(self) -> AsyncConversationsResource:
        return AsyncConversationsResource(self._client)

    @cached_property
    def datasets(self) -> AsyncDatasetsResource:
        return AsyncDatasetsResource(self._client)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResource:
        return AsyncMcpServersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncAgentsResourceWithStreamingResponse(self)


class AgentsResourceWithRawResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

    @cached_property
    def data_summary(self) -> DataSummaryResourceWithRawResponse:
        return DataSummaryResourceWithRawResponse(self._agents.data_summary)

    @cached_property
    def conversations(self) -> ConversationsResourceWithRawResponse:
        return ConversationsResourceWithRawResponse(self._agents.conversations)

    @cached_property
    def datasets(self) -> DatasetsResourceWithRawResponse:
        return DatasetsResourceWithRawResponse(self._agents.datasets)

    @cached_property
    def mcp_servers(self) -> McpServersResourceWithRawResponse:
        return McpServersResourceWithRawResponse(self._agents.mcp_servers)


class AsyncAgentsResourceWithRawResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

    @cached_property
    def data_summary(self) -> AsyncDataSummaryResourceWithRawResponse:
        return AsyncDataSummaryResourceWithRawResponse(self._agents.data_summary)

    @cached_property
    def conversations(self) -> AsyncConversationsResourceWithRawResponse:
        return AsyncConversationsResourceWithRawResponse(self._agents.conversations)

    @cached_property
    def datasets(self) -> AsyncDatasetsResourceWithRawResponse:
        return AsyncDatasetsResourceWithRawResponse(self._agents.datasets)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResourceWithRawResponse:
        return AsyncMcpServersResourceWithRawResponse(self._agents.mcp_servers)


class AgentsResourceWithStreamingResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

    @cached_property
    def data_summary(self) -> DataSummaryResourceWithStreamingResponse:
        return DataSummaryResourceWithStreamingResponse(self._agents.data_summary)

    @cached_property
    def conversations(self) -> ConversationsResourceWithStreamingResponse:
        return ConversationsResourceWithStreamingResponse(self._agents.conversations)

    @cached_property
    def datasets(self) -> DatasetsResourceWithStreamingResponse:
        return DatasetsResourceWithStreamingResponse(self._agents.datasets)

    @cached_property
    def mcp_servers(self) -> McpServersResourceWithStreamingResponse:
        return McpServersResourceWithStreamingResponse(self._agents.mcp_servers)


class AsyncAgentsResourceWithStreamingResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

    @cached_property
    def data_summary(self) -> AsyncDataSummaryResourceWithStreamingResponse:
        return AsyncDataSummaryResourceWithStreamingResponse(self._agents.data_summary)

    @cached_property
    def conversations(self) -> AsyncConversationsResourceWithStreamingResponse:
        return AsyncConversationsResourceWithStreamingResponse(self._agents.conversations)

    @cached_property
    def datasets(self) -> AsyncDatasetsResourceWithStreamingResponse:
        return AsyncDatasetsResourceWithStreamingResponse(self._agents.datasets)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResourceWithStreamingResponse:
        return AsyncMcpServersResourceWithStreamingResponse(self._agents.mcp_servers)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ConversationCreateParams"]


class ConversationCreateParams(TypedDict, total=False):
    user_identifier: Required[Annotated[str, PropertyInfo(alias="userIdentifier")]]
    """
    Unique identifier for the end-user (1-256 chars). Allowed characters:
    alphanumeric, underscore, hyphen, period, at symbol.
    """

    user_context: Annotated[Dict[str, Union[str, float, bool]], PropertyInfo(alias="userContext")]
    """
    Optional context key-values for additional filtering/tenancy. Required when User
    Context status is ENABLED. Must be omitted when User Context status is DISABLED.
    Requests fail when User Context is UNSET.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .response import (
    ResponseResource,
    AsyncResponseResource,
    ResponseResourceWithRawResponse,
    AsyncResponseResourceWithRawResponse,
    ResponseResourceWithStreamingResponse,
    AsyncResponseResourceWithStreamingResponse,
)
from .conversations import (
    ConversationsResource,
    AsyncConversationsResource,
    ConversationsResourceWithRawResponse,
    AsyncConversationsResourceWithRawResponse,
    ConversationsResourceWithStreamingResponse,
    AsyncConversationsResourceWithStreamingResponse,
)

__all__ = [
    "ResponseResource",
    "AsyncResponseResource",
    "ResponseResourceWithRawResponse",
    "AsyncResponseResourceWithRawResponse",
    "ResponseResourceWithStreamingResponse",
    "AsyncResponseResourceWithStreamingResponse",
    "ConversationsResource",
    "AsyncConversationsResource",
    "ConversationsResourceWithRawResponse",
    "AsyncConversationsResourceWithRawResponse",
    "ConversationsResourceWithStreamingResponse",
    "AsyncConversationsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .inconvo_conversation import InconvoConversation as InconvoConversation
from .conversation_list_params import ConversationListParams as ConversationListParams
from .conversation_create_params import ConversationCreateParams as ConversationCreateParams
from .conversation_list_response import ConversationListResponse as ConversationListResponse
from .conversation_create_response import ConversationCreateResponse as ConversationCreateResponse
from .data_summary_retrieve_response import DataSummaryRetrieveResponse as DataSummaryRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ContextDeleteResponse"]


class ContextDeleteResponse(BaseModel):
    file: str

    success: bool

    error: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["TenantCreateParams"]


class TenantCreateParams(TypedDict, total=False):
    agent_id: Required[Annotated[str, PropertyInfo(alias="agentId")]]

    tenant: Required[Dict[str, Dict[str, object]]]
    """Arbitrary key-value mapping.

    - Keys are strings.
    - Values are JSON objects with arbitrary properties.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tenant_create_params import TenantCreateParams as TenantCreateParams
from .tenant_create_response import TenantCreateResponse as TenantCreateResponse
from .tenant_delete_response import TenantDeleteResponse as TenantDeleteResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tenants import (
    TenantsResource,
    AsyncTenantsResource,
    TenantsResourceWithRawResponse,
    AsyncTenantsResourceWithRawResponse,
    TenantsResourceWithStreamingResponse,
    AsyncTenantsResourceWithStreamingResponse,
)
from .mcp_servers import (
    McpServersResource,
    AsyncMcpServersResource,
    McpServersResourceWithRawResponse,
    AsyncMcpServersResourceWithRawResponse,
    McpServersResourceWithStreamingResponse,
    AsyncMcpServersResourceWithStreamingResponse,
)

__all__ = [
    "TenantsResource",
    "AsyncTenantsResource",
    "TenantsResourceWithRawResponse",
    "AsyncTenantsResourceWithRawResponse",
    "TenantsResourceWithStreamingResponse",
    "AsyncTenantsResourceWithStreamingResponse",
    "McpServersResource",
    "AsyncMcpServersResource",
    "McpServersResourceWithRawResponse",
    "AsyncMcpServersResourceWithRawResponse",
    "McpServersResourceWithStreamingResponse",
    "AsyncMcpServersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.agents.data_summary_retrieve_response import DataSummaryRetrieveResponse

__all__ = ["DataSummaryResource", "AsyncDataSummaryResource"]


class DataSummaryResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DataSummaryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return DataSummaryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataSummaryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return DataSummaryResourceWithStreamingResponse(self)

    def retrieve(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DataSummaryRetrieveResponse:
        """
        Retrieve a summary of data available to the agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get(
            f"/agents/{agent_id}/data-summary",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataSummaryRetrieveResponse,
        )


class AsyncDataSummaryResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDataSummaryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDataSummaryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataSummaryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncDataSummaryResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DataSummaryRetrieveResponse:
        """
        Retrieve a summary of data available to the agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._get(
            f"/agents/{agent_id}/data-summary",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataSummaryRetrieveResponse,
        )


class DataSummaryResourceWithRawResponse:
    def __init__(self, data_summary: DataSummaryResource) -> None:
        self._data_summary = data_summary

        self.retrieve = to_raw_response_wrapper(
            data_summary.retrieve,
        )


class AsyncDataSummaryResourceWithRawResponse:
    def __init__(self, data_summary: AsyncDataSummaryResource) -> None:
        self._data_summary = data_summary

        self.retrieve = async_to_raw_response_wrapper(
            data_summary.retrieve,
        )


class DataSummaryResourceWithStreamingResponse:
    def __init__(self, data_summary: DataSummaryResource) -> None:
        self._data_summary = data_summary

        self.retrieve = to_streamed_response_wrapper(
            data_summary.retrieve,
        )


class AsyncDataSummaryResourceWithStreamingResponse:
    def __init__(self, data_summary: AsyncDataSummaryResource) -> None:
        self._data_summary = data_summary

        self.retrieve = async_to_streamed_response_wrapper(
            data_summary.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["Feedback"]


class Feedback(BaseModel):
    id: str

    rating: Literal["positive", "negative"]

    comment: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .tenants import (
    TenantsResource,
    AsyncTenantsResource,
    TenantsResourceWithRawResponse,
    AsyncTenantsResourceWithRawResponse,
    TenantsResourceWithStreamingResponse,
    AsyncTenantsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["McpServersResource", "AsyncMcpServersResource"]


class McpServersResource(SyncAPIResource):
    @cached_property
    def tenants(self) -> TenantsResource:
        return TenantsResource(self._client)

    @cached_property
    def with_raw_response(self) -> McpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return McpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> McpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return McpServersResourceWithStreamingResponse(self)


class AsyncMcpServersResource(AsyncAPIResource):
    @cached_property
    def tenants(self) -> AsyncTenantsResource:
        return AsyncTenantsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMcpServersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncMcpServersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMcpServersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncMcpServersResourceWithStreamingResponse(self)


class McpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

    @cached_property
    def tenants(self) -> TenantsResourceWithRawResponse:
        return TenantsResourceWithRawResponse(self._mcp_servers.tenants)


class AsyncMcpServersResourceWithRawResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

    @cached_property
    def tenants(self) -> AsyncTenantsResourceWithRawResponse:
        return AsyncTenantsResourceWithRawResponse(self._mcp_servers.tenants)


class McpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: McpServersResource) -> None:
        self._mcp_servers = mcp_servers

    @cached_property
    def tenants(self) -> TenantsResourceWithStreamingResponse:
        return TenantsResourceWithStreamingResponse(self._mcp_servers.tenants)


class AsyncMcpServersResourceWithStreamingResponse:
    def __init__(self, mcp_servers: AsyncMcpServersResource) -> None:
        self._mcp_servers = mcp_servers

    @cached_property
    def tenants(self) -> AsyncTenantsResourceWithStreamingResponse:
        return AsyncTenantsResourceWithStreamingResponse(self._mcp_servers.tenants)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from inconvo import Inconvo, AsyncInconvo
from tests.utils import assert_matches_type
from inconvo.types.agents.mcp_servers import TenantCreateResponse, TenantDeleteResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTenants:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Inconvo) -> None:
        tenant = client.agents.mcp_servers.tenants.create(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant={"fromapi@api.com": {"organisationId": "bar"}},
        )
        assert_matches_type(TenantCreateResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Inconvo) -> None:
        response = client.agents.mcp_servers.tenants.with_raw_response.create(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant={"fromapi@api.com": {"organisationId": "bar"}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = response.parse()
        assert_matches_type(TenantCreateResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Inconvo) -> None:
        with client.agents.mcp_servers.tenants.with_streaming_response.create(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant={"fromapi@api.com": {"organisationId": "bar"}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = response.parse()
            assert_matches_type(TenantCreateResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Inconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.agents.mcp_servers.tenants.with_raw_response.create(
                mcpserver_id="mcpserver_id",
                agent_id="",
                tenant={"fromapi@api.com": {"organisationId": "bar"}},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcpserver_id` but received ''"):
            client.agents.mcp_servers.tenants.with_raw_response.create(
                mcpserver_id="",
                agent_id="agentId",
                tenant={"fromapi@api.com": {"organisationId": "bar"}},
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Inconvo) -> None:
        tenant = client.agents.mcp_servers.tenants.delete(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant_key="tenant_key",
        )
        assert_matches_type(TenantDeleteResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Inconvo) -> None:
        response = client.agents.mcp_servers.tenants.with_raw_response.delete(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant_key="tenant_key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = response.parse()
        assert_matches_type(TenantDeleteResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Inconvo) -> None:
        with client.agents.mcp_servers.tenants.with_streaming_response.delete(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant_key="tenant_key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = response.parse()
            assert_matches_type(TenantDeleteResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Inconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.agents.mcp_servers.tenants.with_raw_response.delete(
                mcpserver_id="mcpserver_id",
                agent_id="",
                tenant_key="tenant_key",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcpserver_id` but received ''"):
            client.agents.mcp_servers.tenants.with_raw_response.delete(
                mcpserver_id="",
                agent_id="agentId",
                tenant_key="tenant_key",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_key` but received ''"):
            client.agents.mcp_servers.tenants.with_raw_response.delete(
                mcpserver_id="mcpserver_id",
                agent_id="agentId",
                tenant_key="",
            )


class TestAsyncTenants:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncInconvo) -> None:
        tenant = await async_client.agents.mcp_servers.tenants.create(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant={"fromapi@api.com": {"organisationId": "bar"}},
        )
        assert_matches_type(TenantCreateResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncInconvo) -> None:
        response = await async_client.agents.mcp_servers.tenants.with_raw_response.create(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant={"fromapi@api.com": {"organisationId": "bar"}},
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = await response.parse()
        assert_matches_type(TenantCreateResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncInconvo) -> None:
        async with async_client.agents.mcp_servers.tenants.with_streaming_response.create(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant={"fromapi@api.com": {"organisationId": "bar"}},
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = await response.parse()
            assert_matches_type(TenantCreateResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncInconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.agents.mcp_servers.tenants.with_raw_response.create(
                mcpserver_id="mcpserver_id",
                agent_id="",
                tenant={"fromapi@api.com": {"organisationId": "bar"}},
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcpserver_id` but received ''"):
            await async_client.agents.mcp_servers.tenants.with_raw_response.create(
                mcpserver_id="",
                agent_id="agentId",
                tenant={"fromapi@api.com": {"organisationId": "bar"}},
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncInconvo) -> None:
        tenant = await async_client.agents.mcp_servers.tenants.delete(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant_key="tenant_key",
        )
        assert_matches_type(TenantDeleteResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncInconvo) -> None:
        response = await async_client.agents.mcp_servers.tenants.with_raw_response.delete(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant_key="tenant_key",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tenant = await response.parse()
        assert_matches_type(TenantDeleteResponse, tenant, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncInconvo) -> None:
        async with async_client.agents.mcp_servers.tenants.with_streaming_response.delete(
            mcpserver_id="mcpserver_id",
            agent_id="agentId",
            tenant_key="tenant_key",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tenant = await response.parse()
            assert_matches_type(TenantDeleteResponse, tenant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncInconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.agents.mcp_servers.tenants.with_raw_response.delete(
                mcpserver_id="mcpserver_id",
                agent_id="",
                tenant_key="tenant_key",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `mcpserver_id` but received ''"):
            await async_client.agents.mcp_servers.tenants.with_raw_response.delete(
                mcpserver_id="",
                agent_id="agentId",
                tenant_key="tenant_key",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `tenant_key` but received ''"):
            await async_client.agents.mcp_servers.tenants.with_raw_response.delete(
                mcpserver_id="mcpserver_id",
                agent_id="agentId",
                tenant_key="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["TenantDeleteResponse"]


class TenantDeleteResponse(BaseModel):
    success: bool

    tenant_key: str = FieldInfo(alias="tenantKey")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .feedback import (
    FeedbackResource,
    AsyncFeedbackResource,
    FeedbackResourceWithRawResponse,
    AsyncFeedbackResourceWithRawResponse,
    FeedbackResourceWithStreamingResponse,
    AsyncFeedbackResourceWithStreamingResponse,
)
from .response import (
    ResponseResource,
    AsyncResponseResource,
    ResponseResourceWithRawResponse,
    AsyncResponseResourceWithRawResponse,
    ResponseResourceWithStreamingResponse,
    AsyncResponseResourceWithStreamingResponse,
)

__all__ = [
    "FeedbackResource",
    "AsyncFeedbackResource",
    "FeedbackResourceWithRawResponse",
    "AsyncFeedbackResourceWithRawResponse",
    "FeedbackResourceWithStreamingResponse",
    "AsyncFeedbackResourceWithStreamingResponse",
    "ResponseResource",
    "AsyncResponseResource",
    "ResponseResourceWithRawResponse",
    "AsyncResponseResourceWithRawResponse",
    "ResponseResourceWithStreamingResponse",
    "AsyncResponseResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from ...._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.agents.datasets import user_upload_params
from ....types.agents.datasets.user_list_response import UserListResponse
from ....types.agents.datasets.user_delete_response import UserDeleteResponse
from ....types.agents.datasets.user_upload_response import UserUploadResponse

__all__ = ["UserResource", "AsyncUserResource"]


class UserResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return UserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return UserResourceWithStreamingResponse(self)

    def list(
        self,
        user_identifier: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListResponse:
        """
        List all dataset files scoped to a specific user.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not user_identifier:
            raise ValueError(f"Expected a non-empty value for `user_identifier` but received {user_identifier!r}")
        return self._get(
            f"/agents/{agent_id}/datasets/user/{user_identifier}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserListResponse,
        )

    def delete(
        self,
        filename: str,
        *,
        agent_id: str,
        user_identifier: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserDeleteResponse:
        """
        Delete a dataset file scoped to a specific user.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not user_identifier:
            raise ValueError(f"Expected a non-empty value for `user_identifier` but received {user_identifier!r}")
        if not filename:
            raise ValueError(f"Expected a non-empty value for `filename` but received {filename!r}")
        return self._delete(
            f"/agents/{agent_id}/datasets/user/{user_identifier}/{filename}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserDeleteResponse,
        )

    def upload(
        self,
        user_identifier: str,
        *,
        agent_id: str,
        file: FileTypes,
        notes: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserUploadResponse:
        """
        Upload a dataset file scoped to a specific user.

        Args:
          file: The file to upload (CSV or JSON, max 10MB)

          notes: Optional notes or description for the dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not user_identifier:
            raise ValueError(f"Expected a non-empty value for `user_identifier` but received {user_identifier!r}")
        body = deepcopy_minimal(
            {
                "file": file,
                "notes": notes,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/agents/{agent_id}/datasets/user/{user_identifier}",
            body=maybe_transform(body, user_upload_params.UserUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserUploadResponse,
        )


class AsyncUserResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncUserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncUserResourceWithStreamingResponse(self)

    async def list(
        self,
        user_identifier: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListResponse:
        """
        List all dataset files scoped to a specific user.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not user_identifier:
            raise ValueError(f"Expected a non-empty value for `user_identifier` but received {user_identifier!r}")
        return await self._get(
            f"/agents/{agent_id}/datasets/user/{user_identifier}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserListResponse,
        )

    async def delete(
        self,
        filename: str,
        *,
        agent_id: str,
        user_identifier: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserDeleteResponse:
        """
        Delete a dataset file scoped to a specific user.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not user_identifier:
            raise ValueError(f"Expected a non-empty value for `user_identifier` but received {user_identifier!r}")
        if not filename:
            raise ValueError(f"Expected a non-empty value for `filename` but received {filename!r}")
        return await self._delete(
            f"/agents/{agent_id}/datasets/user/{user_identifier}/{filename}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserDeleteResponse,
        )

    async def upload(
        self,
        user_identifier: str,
        *,
        agent_id: str,
        file: FileTypes,
        notes: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserUploadResponse:
        """
        Upload a dataset file scoped to a specific user.

        Args:
          file: The file to upload (CSV or JSON, max 10MB)

          notes: Optional notes or description for the dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not user_identifier:
            raise ValueError(f"Expected a non-empty value for `user_identifier` but received {user_identifier!r}")
        body = deepcopy_minimal(
            {
                "file": file,
                "notes": notes,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/agents/{agent_id}/datasets/user/{user_identifier}",
            body=await async_maybe_transform(body, user_upload_params.UserUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserUploadResponse,
        )


class UserResourceWithRawResponse:
    def __init__(self, user: UserResource) -> None:
        self._user = user

        self.list = to_raw_response_wrapper(
            user.list,
        )
        self.delete = to_raw_response_wrapper(
            user.delete,
        )
        self.upload = to_raw_response_wrapper(
            user.upload,
        )


class AsyncUserResourceWithRawResponse:
    def __init__(self, user: AsyncUserResource) -> None:
        self._user = user

        self.list = async_to_raw_response_wrapper(
            user.list,
        )
        self.delete = async_to_raw_response_wrapper(
            user.delete,
        )
        self.upload = async_to_raw_response_wrapper(
            user.upload,
        )


class UserResourceWithStreamingResponse:
    def __init__(self, user: UserResource) -> None:
        self._user = user

        self.list = to_streamed_response_wrapper(
            user.list,
        )
        self.delete = to_streamed_response_wrapper(
            user.delete,
        )
        self.upload = to_streamed_response_wrapper(
            user.upload,
        )


class AsyncUserResourceWithStreamingResponse:
    def __init__(self, user: AsyncUserResource) -> None:
        self._user = user

        self.list = async_to_streamed_response_wrapper(
            user.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            user.delete,
        )
        self.upload = async_to_streamed_response_wrapper(
            user.upload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ConversationListParams"]


class ConversationListParams(TypedDict, total=False):
    cursor: str

    limit: int

    user_context: Annotated[Dict[str, Union[str, float, bool]], PropertyInfo(alias="userContext")]
    """
    Arbitrary userContext filters, encoded as a deep object. Example: GET
    /conversations?userContext[userId]=42&userContext[orgId]=12
    """

    user_identifier: Annotated[str, PropertyInfo(alias="userIdentifier")]
    """Filter by user identifier"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .agents import (
    AgentsResource,
    AsyncAgentsResource,
    AgentsResourceWithRawResponse,
    AsyncAgentsResourceWithRawResponse,
    AgentsResourceWithStreamingResponse,
    AsyncAgentsResourceWithStreamingResponse,
)
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .mcp_servers import (
    McpServersResource,
    AsyncMcpServersResource,
    McpServersResourceWithRawResponse,
    AsyncMcpServersResourceWithRawResponse,
    McpServersResourceWithStreamingResponse,
    AsyncMcpServersResourceWithStreamingResponse,
)
from .data_summary import (
    DataSummaryResource,
    AsyncDataSummaryResource,
    DataSummaryResourceWithRawResponse,
    AsyncDataSummaryResourceWithRawResponse,
    DataSummaryResourceWithStreamingResponse,
    AsyncDataSummaryResourceWithStreamingResponse,
)
from .conversations import (
    ConversationsResource,
    AsyncConversationsResource,
    ConversationsResourceWithRawResponse,
    AsyncConversationsResourceWithRawResponse,
    ConversationsResourceWithStreamingResponse,
    AsyncConversationsResourceWithStreamingResponse,
)

__all__ = [
    "DataSummaryResource",
    "AsyncDataSummaryResource",
    "DataSummaryResourceWithRawResponse",
    "AsyncDataSummaryResourceWithRawResponse",
    "DataSummaryResourceWithStreamingResponse",
    "AsyncDataSummaryResourceWithStreamingResponse",
    "ConversationsResource",
    "AsyncConversationsResource",
    "ConversationsResourceWithRawResponse",
    "AsyncConversationsResourceWithRawResponse",
    "ConversationsResourceWithStreamingResponse",
    "AsyncConversationsResourceWithStreamingResponse",
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
    "McpServersResource",
    "AsyncMcpServersResource",
    "McpServersResourceWithRawResponse",
    "AsyncMcpServersResourceWithRawResponse",
    "McpServersResourceWithStreamingResponse",
    "AsyncMcpServersResourceWithStreamingResponse",
    "AgentsResource",
    "AsyncAgentsResource",
    "AgentsResourceWithRawResponse",
    "AsyncAgentsResourceWithRawResponse",
    "AgentsResourceWithStreamingResponse",
    "AsyncAgentsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["UserUploadResponse", "File"]


class File(BaseModel):
    name: str

    path: str

    size: int


class UserUploadResponse(BaseModel):
    file: File

    error: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["DataSummaryRetrieveResponse"]


class DataSummaryRetrieveResponse(BaseModel):
    data_summary: str = FieldInfo(alias="dataSummary")
    """A human-readable summary of data available to the agent"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, cast

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, FileTypes, omit, not_given
from ...._utils import extract_files, maybe_transform, deepcopy_minimal, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.agents.datasets import context_upload_params
from ....types.agents.datasets.context_list_response import ContextListResponse
from ....types.agents.datasets.context_delete_response import ContextDeleteResponse
from ....types.agents.datasets.context_upload_response import ContextUploadResponse

__all__ = ["ContextResource", "AsyncContextResource"]


class ContextResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ContextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ContextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ContextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return ContextResourceWithStreamingResponse(self)

    def list(
        self,
        context_value: str,
        *,
        agent_id: str,
        context_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContextListResponse:
        """List all dataset files scoped to a context value.

        These files are shared with
        all users who have this context value. Requires User Context to be enabled.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not context_key:
            raise ValueError(f"Expected a non-empty value for `context_key` but received {context_key!r}")
        if not context_value:
            raise ValueError(f"Expected a non-empty value for `context_value` but received {context_value!r}")
        return self._get(
            f"/agents/{agent_id}/datasets/context/{context_key}/{context_value}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContextListResponse,
        )

    def delete(
        self,
        filename: str,
        *,
        agent_id: str,
        context_key: str,
        context_value: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContextDeleteResponse:
        """Delete a dataset file scoped to a context value.

        Requires User Context to be
        enabled.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not context_key:
            raise ValueError(f"Expected a non-empty value for `context_key` but received {context_key!r}")
        if not context_value:
            raise ValueError(f"Expected a non-empty value for `context_value` but received {context_value!r}")
        if not filename:
            raise ValueError(f"Expected a non-empty value for `filename` but received {filename!r}")
        return self._delete(
            f"/agents/{agent_id}/datasets/context/{context_key}/{context_value}/{filename}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContextDeleteResponse,
        )

    def upload(
        self,
        context_value: str,
        *,
        agent_id: str,
        context_key: str,
        file: FileTypes,
        notes: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContextUploadResponse:
        """Upload a dataset file scoped to a context value.

        This file will be shared with
        all users who have this context value. Requires User Context to be enabled and
        contextKey to be defined in the schema.

        Args:
          file: The file to upload (CSV or JSON, max 10MB)

          notes: Optional notes or description for the dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not context_key:
            raise ValueError(f"Expected a non-empty value for `context_key` but received {context_key!r}")
        if not context_value:
            raise ValueError(f"Expected a non-empty value for `context_value` but received {context_value!r}")
        body = deepcopy_minimal(
            {
                "file": file,
                "notes": notes,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            f"/agents/{agent_id}/datasets/context/{context_key}/{context_value}",
            body=maybe_transform(body, context_upload_params.ContextUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContextUploadResponse,
        )


class AsyncContextResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncContextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncContextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncContextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncContextResourceWithStreamingResponse(self)

    async def list(
        self,
        context_value: str,
        *,
        agent_id: str,
        context_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContextListResponse:
        """List all dataset files scoped to a context value.

        These files are shared with
        all users who have this context value. Requires User Context to be enabled.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not context_key:
            raise ValueError(f"Expected a non-empty value for `context_key` but received {context_key!r}")
        if not context_value:
            raise ValueError(f"Expected a non-empty value for `context_value` but received {context_value!r}")
        return await self._get(
            f"/agents/{agent_id}/datasets/context/{context_key}/{context_value}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContextListResponse,
        )

    async def delete(
        self,
        filename: str,
        *,
        agent_id: str,
        context_key: str,
        context_value: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContextDeleteResponse:
        """Delete a dataset file scoped to a context value.

        Requires User Context to be
        enabled.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not context_key:
            raise ValueError(f"Expected a non-empty value for `context_key` but received {context_key!r}")
        if not context_value:
            raise ValueError(f"Expected a non-empty value for `context_value` but received {context_value!r}")
        if not filename:
            raise ValueError(f"Expected a non-empty value for `filename` but received {filename!r}")
        return await self._delete(
            f"/agents/{agent_id}/datasets/context/{context_key}/{context_value}/{filename}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContextDeleteResponse,
        )

    async def upload(
        self,
        context_value: str,
        *,
        agent_id: str,
        context_key: str,
        file: FileTypes,
        notes: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ContextUploadResponse:
        """Upload a dataset file scoped to a context value.

        This file will be shared with
        all users who have this context value. Requires User Context to be enabled and
        contextKey to be defined in the schema.

        Args:
          file: The file to upload (CSV or JSON, max 10MB)

          notes: Optional notes or description for the dataset

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not context_key:
            raise ValueError(f"Expected a non-empty value for `context_key` but received {context_key!r}")
        if not context_value:
            raise ValueError(f"Expected a non-empty value for `context_value` but received {context_value!r}")
        body = deepcopy_minimal(
            {
                "file": file,
                "notes": notes,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            f"/agents/{agent_id}/datasets/context/{context_key}/{context_value}",
            body=await async_maybe_transform(body, context_upload_params.ContextUploadParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ContextUploadResponse,
        )


class ContextResourceWithRawResponse:
    def __init__(self, context: ContextResource) -> None:
        self._context = context

        self.list = to_raw_response_wrapper(
            context.list,
        )
        self.delete = to_raw_response_wrapper(
            context.delete,
        )
        self.upload = to_raw_response_wrapper(
            context.upload,
        )


class AsyncContextResourceWithRawResponse:
    def __init__(self, context: AsyncContextResource) -> None:
        self._context = context

        self.list = async_to_raw_response_wrapper(
            context.list,
        )
        self.delete = async_to_raw_response_wrapper(
            context.delete,
        )
        self.upload = async_to_raw_response_wrapper(
            context.upload,
        )


class ContextResourceWithStreamingResponse:
    def __init__(self, context: ContextResource) -> None:
        self._context = context

        self.list = to_streamed_response_wrapper(
            context.list,
        )
        self.delete = to_streamed_response_wrapper(
            context.delete,
        )
        self.upload = to_streamed_response_wrapper(
            context.upload,
        )


class AsyncContextResourceWithStreamingResponse:
    def __init__(self, context: AsyncContextResource) -> None:
        self._context = context

        self.list = async_to_streamed_response_wrapper(
            context.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            context.delete,
        )
        self.upload = async_to_streamed_response_wrapper(
            context.upload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["ResponseCreateParams"]


class ResponseCreateParams(TypedDict, total=False):
    agent_id: Required[Annotated[str, PropertyInfo(alias="agentId")]]

    message: Required[str]

    stream: bool
    """
    If true and the client sets `Accept: text/event-stream`, the API returns an SSE
    stream instead of a single JSON body.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["Table"]


class Table(BaseModel):
    body: List[List[str]]

    head: List[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .chart import Chart as Chart
from .table import Table as Table
from .response_create_params import ResponseCreateParams as ResponseCreateParams
from .response_create_response import ResponseCreateResponse as ResponseCreateResponse
from .response_retrieve_response import ResponseRetrieveResponse as ResponseRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from inconvo import Inconvo, AsyncInconvo
from tests.utils import assert_matches_type
from inconvo.types.agents.conversations import ResponseCreateResponse, ResponseRetrieveResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestResponse:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Inconvo) -> None:
        response = client.agents.conversations.response.create(
            conversation_id="conversation_id",
            agent_id="agentId",
            message="message",
        )
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Inconvo) -> None:
        response = client.agents.conversations.response.create(
            conversation_id="conversation_id",
            agent_id="agentId",
            message="message",
            stream=True,
        )
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Inconvo) -> None:
        http_response = client.agents.conversations.response.with_raw_response.create(
            conversation_id="conversation_id",
            agent_id="agentId",
            message="message",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Inconvo) -> None:
        with client.agents.conversations.response.with_streaming_response.create(
            conversation_id="conversation_id",
            agent_id="agentId",
            message="message",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = http_response.parse()
            assert_matches_type(ResponseCreateResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Inconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.agents.conversations.response.with_raw_response.create(
                conversation_id="conversation_id",
                agent_id="",
                message="message",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            client.agents.conversations.response.with_raw_response.create(
                conversation_id="",
                agent_id="agentId",
                message="message",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Inconvo) -> None:
        response = client.agents.conversations.response.retrieve(
            response_id="response_id",
            agent_id="agentId",
            conversation_id="conversation_id",
        )
        assert_matches_type(ResponseRetrieveResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Inconvo) -> None:
        http_response = client.agents.conversations.response.with_raw_response.retrieve(
            response_id="response_id",
            agent_id="agentId",
            conversation_id="conversation_id",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(ResponseRetrieveResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Inconvo) -> None:
        with client.agents.conversations.response.with_streaming_response.retrieve(
            response_id="response_id",
            agent_id="agentId",
            conversation_id="conversation_id",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = http_response.parse()
            assert_matches_type(ResponseRetrieveResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Inconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.agents.conversations.response.with_raw_response.retrieve(
                response_id="response_id",
                agent_id="",
                conversation_id="conversation_id",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            client.agents.conversations.response.with_raw_response.retrieve(
                response_id="response_id",
                agent_id="agentId",
                conversation_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            client.agents.conversations.response.with_raw_response.retrieve(
                response_id="",
                agent_id="agentId",
                conversation_id="conversation_id",
            )


class TestAsyncResponse:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncInconvo) -> None:
        response = await async_client.agents.conversations.response.create(
            conversation_id="conversation_id",
            agent_id="agentId",
            message="message",
        )
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncInconvo) -> None:
        response = await async_client.agents.conversations.response.create(
            conversation_id="conversation_id",
            agent_id="agentId",
            message="message",
            stream=True,
        )
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncInconvo) -> None:
        http_response = await async_client.agents.conversations.response.with_raw_response.create(
            conversation_id="conversation_id",
            agent_id="agentId",
            message="message",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = await http_response.parse()
        assert_matches_type(ResponseCreateResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncInconvo) -> None:
        async with async_client.agents.conversations.response.with_streaming_response.create(
            conversation_id="conversation_id",
            agent_id="agentId",
            message="message",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = await http_response.parse()
            assert_matches_type(ResponseCreateResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncInconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.agents.conversations.response.with_raw_response.create(
                conversation_id="conversation_id",
                agent_id="",
                message="message",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            await async_client.agents.conversations.response.with_raw_response.create(
                conversation_id="",
                agent_id="agentId",
                message="message",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncInconvo) -> None:
        response = await async_client.agents.conversations.response.retrieve(
            response_id="response_id",
            agent_id="agentId",
            conversation_id="conversation_id",
        )
        assert_matches_type(ResponseRetrieveResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncInconvo) -> None:
        http_response = await async_client.agents.conversations.response.with_raw_response.retrieve(
            response_id="response_id",
            agent_id="agentId",
            conversation_id="conversation_id",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = await http_response.parse()
        assert_matches_type(ResponseRetrieveResponse, response, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncInconvo) -> None:
        async with async_client.agents.conversations.response.with_streaming_response.retrieve(
            response_id="response_id",
            agent_id="agentId",
            conversation_id="conversation_id",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = await http_response.parse()
            assert_matches_type(ResponseRetrieveResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncInconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.agents.conversations.response.with_raw_response.retrieve(
                response_id="response_id",
                agent_id="",
                conversation_id="conversation_id",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `conversation_id` but received ''"):
            await async_client.agents.conversations.response.with_raw_response.retrieve(
                response_id="response_id",
                agent_id="agentId",
                conversation_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            await async_client.agents.conversations.response.with_raw_response.retrieve(
                response_id="",
                agent_id="agentId",
                conversation_id="conversation_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.agents.mcp_servers import tenant_create_params
from ....types.agents.mcp_servers.tenant_create_response import TenantCreateResponse
from ....types.agents.mcp_servers.tenant_delete_response import TenantDeleteResponse

__all__ = ["TenantsResource", "AsyncTenantsResource"]


class TenantsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return TenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return TenantsResourceWithStreamingResponse(self)

    def create(
        self,
        mcpserver_id: str,
        *,
        agent_id: str,
        tenant: Dict[str, Dict[str, object]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantCreateResponse:
        """
        Create a tenant for an MCP Server

        Args:
          tenant: Arbitrary key-value mapping.

              - Keys are strings.
              - Values are JSON objects with arbitrary properties.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not mcpserver_id:
            raise ValueError(f"Expected a non-empty value for `mcpserver_id` but received {mcpserver_id!r}")
        return self._post(
            f"/agents/{agent_id}/mcpservers/{mcpserver_id}/tenants",
            body=maybe_transform(tenant, tenant_create_params.TenantCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantCreateResponse,
        )

    def delete(
        self,
        mcpserver_id: str,
        *,
        agent_id: str,
        tenant_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantDeleteResponse:
        """
        Delete a tenant mapping from an MCP Server

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not mcpserver_id:
            raise ValueError(f"Expected a non-empty value for `mcpserver_id` but received {mcpserver_id!r}")
        if not tenant_key:
            raise ValueError(f"Expected a non-empty value for `tenant_key` but received {tenant_key!r}")
        return self._delete(
            f"/agents/{agent_id}/mcpservers/{mcpserver_id}/tenants/{tenant_key}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantDeleteResponse,
        )


class AsyncTenantsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncTenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncTenantsResourceWithStreamingResponse(self)

    async def create(
        self,
        mcpserver_id: str,
        *,
        agent_id: str,
        tenant: Dict[str, Dict[str, object]],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantCreateResponse:
        """
        Create a tenant for an MCP Server

        Args:
          tenant: Arbitrary key-value mapping.

              - Keys are strings.
              - Values are JSON objects with arbitrary properties.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not mcpserver_id:
            raise ValueError(f"Expected a non-empty value for `mcpserver_id` but received {mcpserver_id!r}")
        return await self._post(
            f"/agents/{agent_id}/mcpservers/{mcpserver_id}/tenants",
            body=await async_maybe_transform(tenant, tenant_create_params.TenantCreateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantCreateResponse,
        )

    async def delete(
        self,
        mcpserver_id: str,
        *,
        agent_id: str,
        tenant_key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantDeleteResponse:
        """
        Delete a tenant mapping from an MCP Server

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not mcpserver_id:
            raise ValueError(f"Expected a non-empty value for `mcpserver_id` but received {mcpserver_id!r}")
        if not tenant_key:
            raise ValueError(f"Expected a non-empty value for `tenant_key` but received {tenant_key!r}")
        return await self._delete(
            f"/agents/{agent_id}/mcpservers/{mcpserver_id}/tenants/{tenant_key}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantDeleteResponse,
        )


class TenantsResourceWithRawResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.create = to_raw_response_wrapper(
            tenants.create,
        )
        self.delete = to_raw_response_wrapper(
            tenants.delete,
        )


class AsyncTenantsResourceWithRawResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.create = async_to_raw_response_wrapper(
            tenants.create,
        )
        self.delete = async_to_raw_response_wrapper(
            tenants.delete,
        )


class TenantsResourceWithStreamingResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.create = to_streamed_response_wrapper(
            tenants.create,
        )
        self.delete = to_streamed_response_wrapper(
            tenants.delete,
        )


class AsyncTenantsResourceWithStreamingResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.create = async_to_streamed_response_wrapper(
            tenants.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            tenants.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .user import (
    UserResource,
    AsyncUserResource,
    UserResourceWithRawResponse,
    AsyncUserResourceWithRawResponse,
    UserResourceWithStreamingResponse,
    AsyncUserResourceWithStreamingResponse,
)
from .context import (
    ContextResource,
    AsyncContextResource,
    ContextResourceWithRawResponse,
    AsyncContextResourceWithRawResponse,
    ContextResourceWithStreamingResponse,
    AsyncContextResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["DatasetsResource", "AsyncDatasetsResource"]


class DatasetsResource(SyncAPIResource):
    @cached_property
    def user(self) -> UserResource:
        return UserResource(self._client)

    @cached_property
    def context(self) -> ContextResource:
        return ContextResource(self._client)

    @cached_property
    def with_raw_response(self) -> DatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return DatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return DatasetsResourceWithStreamingResponse(self)


class AsyncDatasetsResource(AsyncAPIResource):
    @cached_property
    def user(self) -> AsyncUserResource:
        return AsyncUserResource(self._client)

    @cached_property
    def context(self) -> AsyncContextResource:
        return AsyncContextResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDatasetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDatasetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatasetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncDatasetsResourceWithStreamingResponse(self)


class DatasetsResourceWithRawResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

    @cached_property
    def user(self) -> UserResourceWithRawResponse:
        return UserResourceWithRawResponse(self._datasets.user)

    @cached_property
    def context(self) -> ContextResourceWithRawResponse:
        return ContextResourceWithRawResponse(self._datasets.context)


class AsyncDatasetsResourceWithRawResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

    @cached_property
    def user(self) -> AsyncUserResourceWithRawResponse:
        return AsyncUserResourceWithRawResponse(self._datasets.user)

    @cached_property
    def context(self) -> AsyncContextResourceWithRawResponse:
        return AsyncContextResourceWithRawResponse(self._datasets.context)


class DatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: DatasetsResource) -> None:
        self._datasets = datasets

    @cached_property
    def user(self) -> UserResourceWithStreamingResponse:
        return UserResourceWithStreamingResponse(self._datasets.user)

    @cached_property
    def context(self) -> ContextResourceWithStreamingResponse:
        return ContextResourceWithStreamingResponse(self._datasets.context)


class AsyncDatasetsResourceWithStreamingResponse:
    def __init__(self, datasets: AsyncDatasetsResource) -> None:
        self._datasets = datasets

    @cached_property
    def user(self) -> AsyncUserResourceWithStreamingResponse:
        return AsyncUserResourceWithStreamingResponse(self._datasets.user)

    @cached_property
    def context(self) -> AsyncContextResourceWithStreamingResponse:
        return AsyncContextResourceWithStreamingResponse(self._datasets.context)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .user_list_response import UserListResponse as UserListResponse
from .user_upload_params import UserUploadParams as UserUploadParams
from .user_delete_response import UserDeleteResponse as UserDeleteResponse
from .user_upload_response import UserUploadResponse as UserUploadResponse
from .context_list_response import ContextListResponse as ContextListResponse
from .context_upload_params import ContextUploadParams as ContextUploadParams
from .context_delete_response import ContextDeleteResponse as ContextDeleteResponse
from .context_upload_response import ContextUploadResponse as ContextUploadResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._types import FileTypes
from ...._utils import PropertyInfo

__all__ = ["ContextUploadParams"]


class ContextUploadParams(TypedDict, total=False):
    agent_id: Required[Annotated[str, PropertyInfo(alias="agentId")]]

    context_key: Required[Annotated[str, PropertyInfo(alias="contextKey")]]

    file: Required[FileTypes]
    """The file to upload (CSV or JSON, max 10MB)"""

    notes: str
    """Optional notes or description for the dataset"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel
from .conversations.chart import Chart
from .conversations.table import Table

__all__ = ["InconvoConversation", "Message"]


class Message(BaseModel):
    message: str

    type: Literal["text", "chart", "table", "error"]

    id: Optional[str] = None
    """Present on Inconvo responses only"""

    chart: Optional[Chart] = None
    """Charts use vega V5 spec https://vega.github.io/schema/vega/v5.json"""

    table: Optional[Table] = None


class InconvoConversation(BaseModel):
    id: str

    context: Optional[Dict[str, Union[str, float, bool]]] = None
    """User context values (null when user context is disabled)"""

    messages: List[Message]

    user_identifier: Optional[str] = FieldInfo(alias="userIdentifier", default=None)
    """Unique identifier for the end-user (may be null for legacy conversations)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .user import (
    UserResource,
    AsyncUserResource,
    UserResourceWithRawResponse,
    AsyncUserResourceWithRawResponse,
    UserResourceWithStreamingResponse,
    AsyncUserResourceWithStreamingResponse,
)
from .context import (
    ContextResource,
    AsyncContextResource,
    ContextResourceWithRawResponse,
    AsyncContextResourceWithRawResponse,
    ContextResourceWithStreamingResponse,
    AsyncContextResourceWithStreamingResponse,
)
from .datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)

__all__ = [
    "UserResource",
    "AsyncUserResource",
    "UserResourceWithRawResponse",
    "AsyncUserResourceWithRawResponse",
    "UserResourceWithStreamingResponse",
    "AsyncUserResourceWithStreamingResponse",
    "ContextResource",
    "AsyncContextResource",
    "ContextResourceWithRawResponse",
    "AsyncContextResourceWithRawResponse",
    "ContextResourceWithStreamingResponse",
    "AsyncContextResourceWithStreamingResponse",
    "DatasetsResource",
    "AsyncDatasetsResource",
    "DatasetsResourceWithRawResponse",
    "AsyncDatasetsResourceWithRawResponse",
    "DatasetsResourceWithStreamingResponse",
    "AsyncDatasetsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .chart import Chart
from .table import Table
from ...._models import BaseModel

__all__ = ["ResponseRetrieveResponse", "Input", "Output", "Trace"]


class Input(BaseModel):
    message: str
    """The input message"""

    user_context: Optional[Dict[str, object]] = FieldInfo(alias="userContext", default=None)
    """Additional context as key-value pairs (null when user context is disabled)"""


class Output(BaseModel):
    text: str
    """The response text"""

    type: Literal["text", "chart", "table", "error"]
    """Type of the output"""

    chart: Optional[Chart] = None
    """Charts use vega V5 spec https://vega.github.io/schema/vega/v5.json"""

    table: Optional[Table] = None


class Trace(BaseModel):
    input: object
    """Input data for the trace step"""

    name: str
    """Name of the trace step"""

    output: object
    """Output data from the trace step"""


class ResponseRetrieveResponse(BaseModel):
    id: str
    """Unique identifier for the response"""

    input: Input

    output: Output

    trace: List[Trace]
    """Array of trace steps"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from inconvo import Inconvo, AsyncInconvo
from tests.utils import assert_matches_type
from inconvo.types.agents.datasets import UserListResponse, UserDeleteResponse, UserUploadResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUser:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Inconvo) -> None:
        user = client.agents.datasets.user.list(
            user_identifier="user_123",
            agent_id="agentId",
        )
        assert_matches_type(UserListResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Inconvo) -> None:
        response = client.agents.datasets.user.with_raw_response.list(
            user_identifier="user_123",
            agent_id="agentId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(UserListResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Inconvo) -> None:
        with client.agents.datasets.user.with_streaming_response.list(
            user_identifier="user_123",
            agent_id="agentId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(UserListResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Inconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.agents.datasets.user.with_raw_response.list(
                user_identifier="user_123",
                agent_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_identifier` but received ''"):
            client.agents.datasets.user.with_raw_response.list(
                user_identifier="",
                agent_id="agentId",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_delete(self, client: Inconvo) -> None:
        user = client.agents.datasets.user.delete(
            filename="data.csv",
            agent_id="agentId",
            user_identifier="user_123",
        )
        assert_matches_type(UserDeleteResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Inconvo) -> None:
        response = client.agents.datasets.user.with_raw_response.delete(
            filename="data.csv",
            agent_id="agentId",
            user_identifier="user_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(UserDeleteResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Inconvo) -> None:
        with client.agents.datasets.user.with_streaming_response.delete(
            filename="data.csv",
            agent_id="agentId",
            user_identifier="user_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(UserDeleteResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Inconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.agents.datasets.user.with_raw_response.delete(
                filename="data.csv",
                agent_id="",
                user_identifier="user_123",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_identifier` but received ''"):
            client.agents.datasets.user.with_raw_response.delete(
                filename="data.csv",
                agent_id="agentId",
                user_identifier="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `filename` but received ''"):
            client.agents.datasets.user.with_raw_response.delete(
                filename="",
                agent_id="agentId",
                user_identifier="user_123",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_upload(self, client: Inconvo) -> None:
        user = client.agents.datasets.user.upload(
            user_identifier="user_123",
            agent_id="agentId",
            file=b"raw file contents",
        )
        assert_matches_type(UserUploadResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_upload_with_all_params(self, client: Inconvo) -> None:
        user = client.agents.datasets.user.upload(
            user_identifier="user_123",
            agent_id="agentId",
            file=b"raw file contents",
            notes="notes",
        )
        assert_matches_type(UserUploadResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_upload(self, client: Inconvo) -> None:
        response = client.agents.datasets.user.with_raw_response.upload(
            user_identifier="user_123",
            agent_id="agentId",
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = response.parse()
        assert_matches_type(UserUploadResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_upload(self, client: Inconvo) -> None:
        with client.agents.datasets.user.with_streaming_response.upload(
            user_identifier="user_123",
            agent_id="agentId",
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = response.parse()
            assert_matches_type(UserUploadResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_upload(self, client: Inconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            client.agents.datasets.user.with_raw_response.upload(
                user_identifier="user_123",
                agent_id="",
                file=b"raw file contents",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_identifier` but received ''"):
            client.agents.datasets.user.with_raw_response.upload(
                user_identifier="",
                agent_id="agentId",
                file=b"raw file contents",
            )


class TestAsyncUser:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncInconvo) -> None:
        user = await async_client.agents.datasets.user.list(
            user_identifier="user_123",
            agent_id="agentId",
        )
        assert_matches_type(UserListResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncInconvo) -> None:
        response = await async_client.agents.datasets.user.with_raw_response.list(
            user_identifier="user_123",
            agent_id="agentId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(UserListResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncInconvo) -> None:
        async with async_client.agents.datasets.user.with_streaming_response.list(
            user_identifier="user_123",
            agent_id="agentId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(UserListResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncInconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.agents.datasets.user.with_raw_response.list(
                user_identifier="user_123",
                agent_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_identifier` but received ''"):
            await async_client.agents.datasets.user.with_raw_response.list(
                user_identifier="",
                agent_id="agentId",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncInconvo) -> None:
        user = await async_client.agents.datasets.user.delete(
            filename="data.csv",
            agent_id="agentId",
            user_identifier="user_123",
        )
        assert_matches_type(UserDeleteResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncInconvo) -> None:
        response = await async_client.agents.datasets.user.with_raw_response.delete(
            filename="data.csv",
            agent_id="agentId",
            user_identifier="user_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(UserDeleteResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncInconvo) -> None:
        async with async_client.agents.datasets.user.with_streaming_response.delete(
            filename="data.csv",
            agent_id="agentId",
            user_identifier="user_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(UserDeleteResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncInconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.agents.datasets.user.with_raw_response.delete(
                filename="data.csv",
                agent_id="",
                user_identifier="user_123",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_identifier` but received ''"):
            await async_client.agents.datasets.user.with_raw_response.delete(
                filename="data.csv",
                agent_id="agentId",
                user_identifier="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `filename` but received ''"):
            await async_client.agents.datasets.user.with_raw_response.delete(
                filename="",
                agent_id="agentId",
                user_identifier="user_123",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_upload(self, async_client: AsyncInconvo) -> None:
        user = await async_client.agents.datasets.user.upload(
            user_identifier="user_123",
            agent_id="agentId",
            file=b"raw file contents",
        )
        assert_matches_type(UserUploadResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_upload_with_all_params(self, async_client: AsyncInconvo) -> None:
        user = await async_client.agents.datasets.user.upload(
            user_identifier="user_123",
            agent_id="agentId",
            file=b"raw file contents",
            notes="notes",
        )
        assert_matches_type(UserUploadResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_upload(self, async_client: AsyncInconvo) -> None:
        response = await async_client.agents.datasets.user.with_raw_response.upload(
            user_identifier="user_123",
            agent_id="agentId",
            file=b"raw file contents",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        user = await response.parse()
        assert_matches_type(UserUploadResponse, user, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_upload(self, async_client: AsyncInconvo) -> None:
        async with async_client.agents.datasets.user.with_streaming_response.upload(
            user_identifier="user_123",
            agent_id="agentId",
            file=b"raw file contents",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            user = await response.parse()
            assert_matches_type(UserUploadResponse, user, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_upload(self, async_client: AsyncInconvo) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
            await async_client.agents.datasets.user.with_raw_response.upload(
                user_identifier="user_123",
                agent_id="",
                file=b"raw file contents",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `user_identifier` but received ''"):
            await async_client.agents.datasets.user.with_raw_response.upload(
                user_identifier="",
                agent_id="agentId",
                file=b"raw file contents",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["FeedbackCreateParams"]


class FeedbackCreateParams(TypedDict, total=False):
    agent_id: Required[Annotated[str, PropertyInfo(alias="agentId")]]

    conversation_id: Required[str]

    rating: Required[Literal["positive", "negative"]]

    comment: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.agents.conversations.response import feedback_create_params, feedback_update_params
from .....types.agents.conversations.response.feedback import Feedback

__all__ = ["FeedbackResource", "AsyncFeedbackResource"]


class FeedbackResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FeedbackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return FeedbackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FeedbackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return FeedbackResourceWithStreamingResponse(self)

    def create(
        self,
        response_id: str,
        *,
        agent_id: str,
        conversation_id: str,
        rating: Literal["positive", "negative"],
        comment: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feedback:
        """
        Create feedback

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        if not response_id:
            raise ValueError(f"Expected a non-empty value for `response_id` but received {response_id!r}")
        return self._post(
            f"/agents/{agent_id}/conversations/{conversation_id}/response/{response_id}/feedback",
            body=maybe_transform(
                {
                    "rating": rating,
                    "comment": comment,
                },
                feedback_create_params.FeedbackCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feedback,
        )

    def update(
        self,
        feedback_id: str,
        *,
        agent_id: str,
        conversation_id: str,
        response_id: str,
        comment: str | Omit = omit,
        rating: Literal["positive", "negative"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feedback:
        """
        Update feedback

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        if not response_id:
            raise ValueError(f"Expected a non-empty value for `response_id` but received {response_id!r}")
        if not feedback_id:
            raise ValueError(f"Expected a non-empty value for `feedback_id` but received {feedback_id!r}")
        return self._patch(
            f"/agents/{agent_id}/conversations/{conversation_id}/response/{response_id}/feedback/{feedback_id}",
            body=maybe_transform(
                {
                    "comment": comment,
                    "rating": rating,
                },
                feedback_update_params.FeedbackUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feedback,
        )


class AsyncFeedbackResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFeedbackResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncFeedbackResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFeedbackResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/inconvoai/inconvo-python-sdk#with_streaming_response
        """
        return AsyncFeedbackResourceWithStreamingResponse(self)

    async def create(
        self,
        response_id: str,
        *,
        agent_id: str,
        conversation_id: str,
        rating: Literal["positive", "negative"],
        comment: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feedback:
        """
        Create feedback

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        if not response_id:
            raise ValueError(f"Expected a non-empty value for `response_id` but received {response_id!r}")
        return await self._post(
            f"/agents/{agent_id}/conversations/{conversation_id}/response/{response_id}/feedback",
            body=await async_maybe_transform(
                {
                    "rating": rating,
                    "comment": comment,
                },
                feedback_create_params.FeedbackCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feedback,
        )

    async def update(
        self,
        feedback_id: str,
        *,
        agent_id: str,
        conversation_id: str,
        response_id: str,
        comment: str | Omit = omit,
        rating: Literal["positive", "negative"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Feedback:
        """
        Update feedback

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not conversation_id:
            raise ValueError(f"Expected a non-empty value for `conversation_id` but received {conversation_id!r}")
        if not response_id:
            raise ValueError(f"Expected a non-empty value for `response_id` but received {response_id!r}")
        if not feedback_id:
            raise ValueError(f"Expected a non-empty value for `feedback_id` but received {feedback_id!r}")
        return await self._patch(
            f"/agents/{agent_id}/conversations/{conversation_id}/response/{response_id}/feedback/{feedback_id}",
            body=await async_maybe_transform(
                {
                    "comment": comment,
                    "rating": rating,
                },
                feedback_update_params.FeedbackUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Feedback,
        )


class FeedbackResourceWithRawResponse:
    def __init__(self, feedback: FeedbackResource) -> None:
        self._feedback = feedback

        self.create = to_raw_response_wrapper(
            feedback.create,
        )
        self.update = to_raw_response_wrapper(
            feedback.update,
        )


class AsyncFeedbackResourceWithRawResponse:
    def __init__(self, feedback: AsyncFeedbackResource) -> None:
        self._feedback = feedback

        self.create = async_to_raw_response_wrapper(
            feedback.create,
        )
        self.update = async_to_raw_response_wrapper(
            feedback.update,
        )


class FeedbackResourceWithStreamingResponse:
    def __init__(self, feedback: FeedbackResource) -> None:
        self._feedback = feedback

        self.create = to_streamed_response_wrapper(
            feedback.create,
        )
        self.update = to_streamed_response_wrapper(
            feedback.update,
        )


class AsyncFeedbackResourceWithStreamingResponse:
    def __init__(self, feedback: AsyncFeedbackResource) -> None:
        self._feedback = feedback

        self.create = async_to_streamed_response_wrapper(
            feedback.create,
        )
        self.update = async_to_streamed_response_wrapper(
            feedback.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .chart import Chart
from .table import Table
from ...._models import BaseModel

__all__ = ["ResponseCreateResponse"]


class ResponseCreateResponse(BaseModel):
    id: str

    conversation_id: str = FieldInfo(alias="conversationId")

    message: str

    type: Literal["text", "chart", "table", "error"]

    chart: Optional[Chart] = None
    """Charts use vega V5 spec https://vega.github.io/schema/vega/v5.json"""

    table: Optional[Table] = None
